import math
from fractions import Fraction

def alpha(x): return math.floor(x + 1/2)

def delta(x): return abs(x - alpha(x))

def tau(n, x):
    return sum([delta(2 ** k * x) / 2 ** k for k in range(n + 1)])

def mod1(x):
    p = x.numerator
    q = x.denominator
    return Fraction(p % q, q)

def f_takagi(x): return mod1(2 * x)

def floyd_takagi(a):
    x = f_takagi(a)
    y = f_takagi(f_takagi(a))
    N = 1
    q = 2
    while delta(y) != delta(x):
        x = f_takagi(x)
        y = f_takagi(f_takagi(y))
        N = N + 1
        q = q + 2
    return (x, N, q - N)

def tau_exact(a):
    (x, N, T) = floyd_takagi(a)
    s1 = 0
    for k in range(N):
        s1 = s1 + Fraction(delta(a), 2 ** k)
        a = f_takagi(a)
    s2 = 0
    for j in range(T):
        s2 = s2 + Fraction(delta(a), 2 ** (N + j))
        a = f_takagi(a)
    return s1 + Fraction(2 ** T, 2 ** T - 1) * s2
